Package: fastbeta
Version: 0.5.1
Date: 2025-10-25
Title: Fast Approximation of Time-Varying Infectious Disease
        Transmission Rates
Description: 
	A fast method for approximating time-varying infectious disease
	transmission rates from disease incidence time series and other
	data, based on a discrete time approximation of an SEIR model, as
	analyzed in Jagan et al. (2020) <doi:10.1371/journal.pcbi.1008124>.
License: GPL (>= 2)
URL: https://github.com/davidearn/fastbeta
BugReports: https://github.com/davidearn/fastbeta/issues
Authors@R: 
	c(person("Mikael", "Jagan",
	         email = "jaganmn@mcmaster.ca",
	         role = c("aut", "cre"),
	         comment = c(ORCID = "0000-0002-3542-2938")),
	  person("David", "Earn", role = "ctb",
	         comment = c(ORCID = "0000-0003-3597-617X")))
Depends: R (>= 4.3)
Imports: grDevices, graphics, stats
Suggests: adaptivetau, deSolve, tools, utils
BuildResaveData: no
NeedsCompilation: yes
Packaged: 2025-10-25 23:41:59 UTC; mikael
Author: Mikael Jagan [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3542-2938>),
  David Earn [ctb] (ORCID: <https://orcid.org/0000-0003-3597-617X>)
Maintainer: Mikael Jagan <jaganmn@mcmaster.ca>
Repository: CRAN
Date/Publication: 2025-10-26 19:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 17:50:11 UTC; unix
Archs: fastbeta.so.dSYM
